@@ -11,13 +11,6 @@ from kamzik3.devices.device import Device
 from kamzik3.snippets.snippetLogging import set_rotating_file_handler, set_file_handler
 from kamzik3.snippets.snippetsTimer import PreciseCallbackTimer
 
-ATTR_LOGFILE = "Logfile"
-ATTR_HEADER = "Header"
-ATTR_LAST_LOG_LINE = "Last log line"
-ATTR_INTERVAL = "Interval"
-ATTR_LOGGING = "Logging"
-
-
 class AttributeLogger(Device):
     # Logging interval in ms
     interval = 1e3
