@@ -33,8 +33,9 @@ Smaract0_axis_0: &Smaract0_axis_0 !Device:kamzik3.devices.deviceSmaractMcs1.Devi
 
 
 class DeviceSmaractMcs1(DeviceSocket):
-    push_buffer_size = 50
+    # push_buffer_size = 50
     response_timeout = 3000
+    push_commands_max = 1
     terminator = b"\n"
     handtool_modes = frozenbidict({"Disabled": 0, "Enabled": 1, "Display only": 2})
     sensor_modes = frozenbidict({"Disabled": 0, "Enabled": 1, "Power safe": 2})
@@ -92,16 +93,13 @@ class DeviceSmaractMcs1(DeviceSocket):
         if output[:2] == ":E":
             channel, error_number = [int(v) for v in output[2:].split(",")]
             if error_number > 0:
-                # if error_number == 129:
-                #     self.set_raw_value(ATTR_SENSOR_PRESENT, False)
-                #     self.set_raw_value(ATTR_SENSOR_MODE, "Disabled")
-                # for channel in range(self[ATTR_CHANNELS][VALUE]):
-                #     self.notify((channel, ATTR_SENSOR_MODE), "Disabled")
-                #     self.notify((channel, ATTR_SENSOR_PRESENT), False)
+                if error_number == 129:
+                    self.notify((channel, ATTR_SENSOR_MODE), "Disabled")
+                    self.notify((channel, ATTR_SENSOR_PRESENT), False)
                 # elif error_number == 140:
                 #     self.set_raw_value(ATTR_SENSOR_MODE, "Disabled")
-                # for channel in range(self[ATTR_CHANNELS][VALUE]):
-                #     self.notify((channel, ATTR_SENSOR_MODE), "Disabled")
+                #     for channel in range(self[ATTR_CHANNELS][VALUE]):
+                #         self.notify((channel, ATTR_SENSOR_MODE), "Disabled")
                 self.handle_command_error(command, output)
         elif command == ":GSI\n":
             self.set_raw_value(ATTR_SERIAL_NUMBER, output.strip(":IN"))
@@ -145,6 +143,7 @@ class DeviceSmaractMcs1(DeviceSocket):
                     self.notify((channel, ATTR_POSITION_UPPER_LIMIT), (max_position, max_revolution))
                 elif cmd == "PL":
                     min_position, max_position = value.split(",")
+                    min_position, max_position = int(min_position), int(max_position)
                     attribute, value = ATTR_POSITION_LOWER_LIMIT, int(min_position)
                     self.notify((channel, ATTR_POSITION_UPPER_LIMIT), int(max_position))
                 if attribute:
@@ -262,6 +261,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
             self._config_attributes()
             self.get_position_limit_range()
             self.start_polling()
+            self.configured = True
             self.set_status(STATUS_CONFIGURED)
             self.logger.info(u"Device configuration took {} sec.".format(time.time() - start_at))
 
@@ -273,7 +273,6 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
 
     def subject_update(self, key, value, subject):
         super(DeviceSmaractMcs1Channel, self).subject_update(key, value, subject)
-
         if self.connected and isinstance(key, tuple) and key[0] == self.channel:
             attribute = key[1]
             if attribute == ATTR_STATE and self.get_value(ATTR_STATE) != value:
@@ -300,14 +299,14 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                 self.get_position_limit_range(value)
             elif attribute == ATTR_SENSOR_PRESENT:
                 if not value:
-                    self.remove_poll_command(":GP{}\n".format(self.channel), 200)
-                    self.remove_poll_command(":GA{}\n".format(self.channel), 200)
+                    self.remove_poll_command(":GP{}\n".format(self.channel), 250)
+                    self.remove_poll_command(":GA{}\n".format(self.channel), 250)
                 else:
                     self._set_position_polling(self[ATTR_SENSOR_TYPE][VALUE])
             elif attribute == ATTR_SENSOR_MODE:
                 if value == "Disabled":
-                    self.remove_poll_command(":GP{}\n".format(self.channel), 200)
-                    self.remove_poll_command(":GA{}\n".format(self.channel), 200)
+                    self.remove_poll_command(":GP{}\n".format(self.channel), 250)
+                    self.remove_poll_command(":GA{}\n".format(self.channel), 250)
                 else:
                     self._set_position_polling(self[ATTR_SENSOR_TYPE][VALUE])
             elif attribute == ATTR_POSITION and self[ATTR_REVOLUTIONS][VALUE] != 0:
@@ -352,19 +351,19 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
         if sensor_type is None:
             return
         elif sensor_type in self.linear_sensor_type:
-            self.poll_command(":GP{}\n".format(self.channel), 200)
+            self.poll_command(":GP{}\n".format(self.channel), 250)
             self.set_value(ATTR_REVOLUTIONS, 0)
         else:
-            self.poll_command(":GA{}\n".format(self.channel), 200)
+            self.poll_command(":GA{}\n".format(self.channel), 250)
 
     def start_polling(self):
         super(DeviceSmaractMcs1Channel, self).start_polling()
-        self.poll_command(":GS{}\n".format(self.channel), 200)
+        self.poll_command(":GS{}\n".format(self.channel), 250)
 
     def stop_polling(self):
-        self.remove_poll_command(":GS{}\n".format(self.channel), 200)
-        self.remove_poll_command(":GP{}\n".format(self.channel), 200)
-        self.remove_poll_command(":GA{}\n".format(self.channel), 200)
+        self.remove_poll_command(":GS{}\n".format(self.channel), 250)
+        self.remove_poll_command(":GP{}\n".format(self.channel), 250)
+        self.remove_poll_command(":GA{}\n".format(self.channel), 250)
 
     def _move_absolute(self, value, callback=None):
         self.logger.info(u"Move to position {}".format(value))
@@ -471,8 +470,8 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
     def set_sensor_type(self, value, callback=None):
         self.logger.info(u"Set sensor type to {}".format(value))
         if value != self.get_value(ATTR_SENSOR_TYPE):
-            self.remove_poll_command(":GP{}\n".format(self.channel), 200)
-            self.remove_poll_command(":GA{}\n".format(self.channel), 200)
+            self.remove_poll_command(":GP{}\n".format(self.channel), 250)
+            self.remove_poll_command(":GA{}\n".format(self.channel), 250)
 
             token = self.command(u":SST{},{}\n".format(self.channel, int(value)), callback, with_token=True)
             self.get_sensor_type()
@@ -480,13 +479,17 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
 
     def set_lower_position_limit(self, value, callback=None):
         self.logger.info(u"Set lower position limit to {}".format(value))
-        upper_limit = self[ATTR_POSITION_UPPER_LIMIT][VALUE]
+        # upper_limit = self[ATTR_POSITION_UPPER_LIMIT][VALUE]
+        upper_limit = self.get_raw_value(ATTR_POSITION_UPPER_LIMIT)
         # self.set_value(ATTR_POSITION_LOWER_LIMIT, int(value))
         if upper_limit is None:
             upper_limit = value
             # self.set_value(ATTR_POSITION_UPPER_LIMIT, upper_limit)
         if int(self[ATTR_SENSOR_TYPE][VALUE]) in self.linear_sensor_type:
-            out = self.command(u":SPL{},{},{}\n".format(self.channel, int(value), int(upper_limit)), with_token=True)
+            if value > upper_limit:
+                value, upper_limit = upper_limit, value
+            out = self.command(u":SPL{},{},{}\n".format(self.channel, int(value), int(upper_limit)),
+                                   with_token=True)
         else:
             revolution = 360 * 10 ** abs(self[ATTR_ENCODER_RESOLUTION][VALUE])
             min_revolutions = int(value // revolution)
@@ -498,20 +501,22 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                                                int(max_angular_position), int(max_revolutions)), callback,
                 with_token=True)
 
-        self.set_raw_value([ATTR_POSITION, MIN], value)
+        self.set_value(ATTR_POSITION, value, key=MIN)
         self.get_position_limit_range()
         return out
 
     def set_upper_position_limit(self, value, callback=None):
         self.logger.info(u"Set upper position limit to {}".format(value))
-        lower_limit = self[ATTR_POSITION_LOWER_LIMIT][VALUE]
+        lower_limit = self.get_raw_value(ATTR_POSITION_LOWER_LIMIT) #self[ATTR_POSITION_LOWER_LIMIT][VALUE]
         # self.set_value(ATTR_POSITION_UPPER_LIMIT, int(value))
         if lower_limit is None:
             lower_limit = value
             # self.set_value(ATTR_POSITION_LOWER_LIMIT, lower_limit)
         if int(self[ATTR_SENSOR_TYPE][VALUE]) in self.linear_sensor_type:
-            out = self.command(u":SPL{},{},{}\n".format(self.channel, int(value), int(lower_limit)), callback,
-                               with_token=True)
+            if value < lower_limit:
+                value, lower_limit = lower_limit, value
+            out = self.command(u":SPL{},{},{}\n".format(self.channel, int(lower_limit), int(value)), callback,
+                                   with_token=True)
         else:
             revolution = 360 * 10 ** abs(self[ATTR_ENCODER_RESOLUTION][VALUE])
             max_revolutions = int(value // revolution)
@@ -523,7 +528,7 @@ class DeviceSmaractMcs1Channel(DeviceChannel):
                                                int(max_angular_position), int(max_revolutions)), callback,
                 with_token=True)
 
-        self.set_raw_value([ATTR_POSITION, MAX], value)
+        self.set_value(ATTR_POSITION, value, key=MAX)
         self.get_position_limit_range()
         return out
 
