@@ -151,10 +151,10 @@ class DeviceSmaractMcs2(DeviceSocket):
             if callback is not None:
                 self.handle_readout_callback(callback, command, readout_buffer)
             if token:
-                self.device_server.push_message(self.device_id, [command, readout_buffer], token)
+                kamzik3.session.publisher.push_message(self.device_id, [command, readout_buffer], token)
                 for observer in self._observers[:]:
                     if isinstance(observer, DeviceSmaractMcs2Channel):
-                        self.device_server.push_message(observer.device_id, [command, readout_buffer], token)
+                        kamzik3.session.publisher.push_message(observer.device_id, [command, readout_buffer], token)
             return
         else:
             command = command.replace(":SYST:ERR:NEXT?\r\n", "")
@@ -260,10 +260,10 @@ class DeviceSmaractMcs2(DeviceSocket):
         if callback is not None:
             self.handle_readout_callback(callback, command, readout_buffer)
         if token:
-            self.device_server.push_message(self.device_id, [command, readout_buffer], token)
+            kamzik3.session.publisher.push_message(self.device_id, [command, readout_buffer], token)
             for observer in self._observers[:]:
                 if isinstance(observer, DeviceSmaractMcs2Channel):
-                    self.device_server.push_message(observer.device_id, [command, readout_buffer], token)
+                    kamzik3.session.publisher.push_message(observer.device_id, [command, readout_buffer], token)
 
     def start_polling(self):
         super(DeviceSmaractMcs2, self).start_polling()
