@@ -233,6 +233,7 @@ class DeviceDebugWidget(Ui_Form, DeviceWidget):
     def _generate_attribute_groups(self, attributes, group, attribute_path):
         column, row = 0, 0
         columns = 4 if self.config is None else self.config.get("columns", 4)
+        is_picture_in = False
         for key, value in attributes.items():
             if Attribute.is_attribute(value):
                 if column % columns == 0:
@@ -242,12 +243,14 @@ class DeviceDebugWidget(Ui_Form, DeviceWidget):
                 attribute_widget.setParent(group)
                 group.layout().addWidget(attribute_widget, row, column)
                 column += 1
+                if value.get(DISPLAY) == IMAGE:
+                    is_picture_in = True
             else:
                 new_group = self._generate_attribute_group(key)
                 self._generate_attribute_groups(value, new_group, attribute_path + [key])
-
-        spacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
-        group.layout().addItem(spacer, row + 1, 0)
+        if not is_picture_in:
+            spacer = QSpacerItem(20, 40, QSizePolicy.Minimum, QSizePolicy.Expanding)
+            group.layout().addItem(spacer, row + 1, 0)
 
         row_count = group.layout().rowCount()
         column_count = group.layout().columnCount()
