@@ -9,10 +9,10 @@ import numpy as np
 import kamzik3
 from kamzik3 import DeviceError, units, MacroException
 from kamzik3.constants import *
-from kamzik3.devices.attribute import Attribute
 from kamzik3.devices.attributeLogger import AttributeLoggerTriggered
 from kamzik3.devices.device import Device
 from kamzik3.macro.common import Common
+from kamzik3.macro.common import StepGenerator
 from kamzik3.macro.scan import Scan
 from kamzik3.macro.step import Step
 from kamzik3.snippets.snippetDataAccess import get_next_file_index
@@ -34,29 +34,26 @@ class Macro(Common, Device):
         self.macro_logger = None
         if self.chain is None:
             self.chain = []
+        self.branch = []
         self.index = -1
         self.macro_body_thread = None
-        self._expose_methods_to_clients()
         self.connect()
 
     def _init_attributes(self):
         super(Macro, self)._init_attributes()
-
-        self.attributes.update({
-            ATTR_STATE: Attribute(IDLE, readonly=True, default_type=np.uint8),
-            ATTR_POINTS_COUNT: Attribute(1, readonly=True, default_type=np.uint64),
-            ATTR_POINT_INDEX: Attribute(0, readonly=True, default_type=np.uint64),
-            ATTR_PROGRESS: Attribute(0, readonly=True, default_type=np.float16, unit="%", min_value=0, max_value=100,
-                                     decimals=2),
-            ATTR_CHAIN_LINK: Attribute(0, readonly=True, default_type=np.uint64),
-            ATTR_CHAIN_LINK_OUTPUT: Attribute("", readonly=True),
-            ATTR_RUNNING_TIME: Attribute(0, readonly=True),
-            ATTR_ESTIMATED_TIME: Attribute(0, readonly=True),
-            ATTR_ESTIMATED_END: Attribute(0, readonly=True),
-            ATTR_REMAINING_TIME: Attribute(0, readonly=True),
-            ATTR_STARTED_AT: Attribute(None, readonly=True),
-            ATTR_OUTPUT_FILENAME: Attribute(None, readonly=True),
-        })
+        self.create_attribute(ATTR_STATE, default_vlaue=IDLE, readonly=True, default_type=np.uint8)
+        self.create_attribute(ATTR_POINTS_COUNT, default_vlaue=1, readonly=True, default_type=np.uint64)
+        self.create_attribute(ATTR_POINT_INDEX, default_vlaue=0, readonly=True, default_type=np.uint64)
+        self.create_attribute(ATTR_PROGRESS, default_vlaue=0, readonly=True, default_type=np.float16, unit="%",
+                              min_value=0, max_value=100, decimals=2)
+        self.create_attribute(ATTR_CHAIN_LINK, default_vlaue=0, readonly=True, default_type=np.uint64)
+        self.create_attribute(ATTR_CHAIN_LINK_OUTPUT, default_vlaue="", readonly=True)
+        self.create_attribute(ATTR_RUNNING_TIME, default_vlaue=0, readonly=True)
+        self.create_attribute(ATTR_ESTIMATED_TIME, default_vlaue=0, readonly=True)
+        self.create_attribute(ATTR_ESTIMATED_END, default_vlaue=0, readonly=True)
+        self.create_attribute(ATTR_REMAINING_TIME, default_vlaue=0, readonly=True)
+        self.create_attribute(ATTR_STARTED_AT, default_vlaue=None, readonly=True)
+        self.create_attribute(ATTR_OUTPUT_FILENAME, default_vlaue=None, readonly=True)
         self.set_value(ATTR_DESCRIPTION, self.comment)
 
     def handle_configuration(self):
@@ -71,6 +68,9 @@ class Macro(Common, Device):
 
         _finish_configuration()
 
+    def command(self, command, callback=None, with_token=False, returning=True):
+        raise DeviceError("Device does not accept any commands.")
+
     def add(self, link):
         if isinstance(link, list):
             for link_item in link:
@@ -116,6 +116,12 @@ class Macro(Common, Device):
         self.chain = new_chain
         self.recount_steps()
 
+    def before_macro_starts(self):
+        pass
+
+    def before_macro_stops(self):
+        pass
+
     @expose_method()
     def start(self, threaded=True):
         if self.get_state() == BUSY:
@@ -130,7 +136,7 @@ class Macro(Common, Device):
         self.set_value(ATTR_ESTIMATED_END, seconds_to_datetime(self.started_at))
         self.current_point_index = 0
         self.index = -1
-
+        self.before_macro_starts()
         if threaded:
             self.macro_body_thread = Thread(target=self.body, args=[self.macro_clean])
             self.macro_body_thread.start()
@@ -139,17 +145,29 @@ class Macro(Common, Device):
 
     def body(self, callback=None):
         try:
+            # Repeat following macro steps
             for repeat in range(self.repeat_count + 1):
+                # Reset all links in chain
                 for link in self.chain:
                     link.reset()
+                # Get first link
                 link = self.next()
+                # Reset branch to empty list
+                self.branch = []
+                # Stay in Macro loop until macro is BUSY
                 while self.get_state() == BUSY:
+                    # Link is empty
                     if link is None:
-                        link = self.prev()
-                        if self.current_point_index == 0:
-                            break
+                        if self.branch:
+                            if self.branch[-1] in self.chain:
+                                link = self.prev()
+                            else:
+                                link = self.branch.pop()
+                                self.index -= 1
                         else:
-                            continue
+                            link = self.prev()
+                        continue
+                    # Link is done and repeated
                     elif link.get_state() == DONE and link.repeated == link.repeat_count:
                         link.reset()
                         link = self.prev()
@@ -160,6 +178,7 @@ class Macro(Common, Device):
                                 self.repeated += 1
                                 break
                         continue
+                    # Deal with Step type of Link
                     elif isinstance(link, Step):
                         if link.get_state() == DONE:
                             link.repeat()
@@ -169,15 +188,17 @@ class Macro(Common, Device):
                             except MacroException as retry_reason:
                                 link.retry(retry_reason)
                                 continue
-                    elif isinstance(link, Scan):
+                    # Deal with StepGenerator type of Link
+                    elif isinstance(link, StepGenerator):
                         if link.as_step:
                             step = link.start()
                             link = step
-                            # step.start()
                             continue
                         # Get step from step generator
                         if link.get_state() == DONE:
                             step = link.repeat()
+                        elif self.get_state() != BUSY:
+                            continue
                         else:
                             try:
                                 step = link.start()
@@ -185,16 +206,26 @@ class Macro(Common, Device):
                                 link.retry(retry_reason)
                                 continue
                         if step is None:
-                            link = self.chain[self.index]
+                            if self.branch:
+                                if link.repeated == link.repeat_count:
+                                    link = self.branch.pop()
+                            else:
+                                link = self.chain[self.index]
                         else:
+                            if link not in self.branch:
+                                self.branch.append(link)
                             link = step
                         continue
 
+                    # Set link index of chain
                     self.set_value(ATTR_CHAIN_LINK, self.index)
+                    # Set output of link
                     self.set_attribute((ATTR_CHAIN_LINK_OUTPUT, VALUE),
                                        [self.current_point_index + 1, self.index, link.get_output()])
+                    # Set point index of macro
                     self.set_point_index(self.current_point_index + 1)
-                    if link.trigger_log and self.macro_logger is not None:
+                    # Write into log file if trigger_log is set and we have logger associated
+                    if link.trigger_log and self.macro_logger:
                         self.macro_logger.trigger()
 
                     if link.repeated == link.repeat_count:
@@ -208,28 +239,29 @@ class Macro(Common, Device):
             self.error(e)
             raise e
         finally:
-            if callback is not None:
+            if callable(callback):
                 callback()
 
     @expose_method()
     def stop(self):
         if self.in_statuses(IDLE_DEVICE_STATUSES):
             raise DeviceError(u"Macro is not running")
-
-        for link in self.chain:
+        for link in self.chain + self.branch:
             link.stop()
+        self.before_macro_stops()
         super(Macro, self).stop()
 
     def macro_clean(self):
         for link in self.chain:
-            if isinstance(link, Scan):
-                link.return_back_step()
+            if isinstance(link, StepGenerator):
+                link.return_back_callback()
+        self.before_macro_stops()
 
     def recount_steps(self):
         sub_steps_count = 0
         sub_points_count = 0
         for link in reversed(self.chain):
-            if isinstance(link, Scan) and sub_steps_count > 0:
+            if isinstance(link, StepGenerator) and sub_steps_count > 0:
                 repeat_step = (link.step_attributes.get("repeat_count", 0) + 1)
                 repeat_scan = (link.repeat_count + 1)
                 total_steps = link.steps_count * sub_steps_count
@@ -263,6 +295,8 @@ class Macro(Common, Device):
         return chain_link_output
 
     def set_state(self, new_state):
+        super(Macro, self).set_state(new_state)
+
         self.set_value(ATTR_STATE, new_state)
         if new_state in (IDLE, DONE, STOPPED):
             self.set_status(STATUS_IDLE)
@@ -274,8 +308,6 @@ class Macro(Common, Device):
         if self.macro_logger is not None and new_state in (DONE, ERROR, IDLE, DONE, STOPPED):
             self.macro_logger.stop()
 
-        super(Macro, self).set_state(new_state)
-
     def set_point_index(self, point_index):
         self.current_point_index = point_index
         self.set_value(ATTR_POINT_INDEX, point_index)
@@ -329,6 +361,8 @@ class Macro(Common, Device):
         return Device.disconnect(self)
 
     def remove(self):
+        for step in self.chain:
+            step.remove()
         Device.remove(self)
         self.macro_logger = None
         self.chain = None
@@ -341,4 +375,4 @@ class Macro(Common, Device):
         self.set_value(ATTR_LAST_ERROR, error_message)
         self.logger.error(error_message)
         self.handle_command_error(self.common_id, error_message)
-        self.set_state(ERROR)
\ No newline at end of file
+        self.set_state(ERROR)
